/*
 * Plasma applet to display application window menus
 *
 * Copyright 2010 Canonical Ltd.
 *
 * Authors:
 * - Aurélien Gâteau <aurelien.gateau@canonical.com>
 *
 * License: GPL v3
 */
#ifndef MENUBARAPPLET_H
#define MENUBARAPPLET_H

// Qt
#include <QCheckBox>
#include <QGraphicsLinearLayout>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsView>
#include <QTimer>

// KDE
#include <Plasma/Applet>
#include <Plasma/ToolButton>

// Local

class Registrar;
class WindowMenuManager;

class MenuButton : public Plasma::ToolButton
{
    Q_OBJECT
public:
    MenuButton(QGraphicsWidget* parent, QMenu* menu)
    : Plasma::ToolButton(parent)
    , mMenu(menu)
    {}

    QMenu* menu() const { return mMenu; }

protected:
    // Let the event propagate so that the applet context menu is shown
    void contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
    {
        event->ignore();
    }

private:
    QMenu* mMenu;
};


class MenuBarApplet : public Plasma::Applet
{
Q_OBJECT
public:
    MenuBarApplet(QObject* parent, const QVariantList& args);
    ~MenuBarApplet();

    virtual void init();

    virtual void createConfigurationInterface(KConfigDialog*);

protected:
    virtual bool eventFilter(QObject*, QEvent*);

private Q_SLOTS:
    void updateMenuBar();
    void slotButtonClicked();
    void showMenu(MenuButton*);
    void slotAboutToHideMenu();
    void checkMousePosition();
    void applyConfig();
    void slotActivated();

private:
    Registrar* mRegistrar;
    QGraphicsLinearLayout* mLayout;
    QTimer* mMouseChecker;
    QMenu* mEmptyMenu;
    WindowMenuManager* mWindowMenuManager;
    QMenu* mWindowMenu;

    QList<MenuButton*> mMenuButtonList;
    MenuButton* mCurrentButton;

    QCheckBox* mConfigButtonFormFactor;

    void createButton(const QString& text, QMenu*);
    void createButtonsForBarFormFactor(QMenu*);
    void createButtonsForButtonFormFactor(QMenu*);
    void updateSizePolicy();

    bool useButtonFormFactor() const;

    void showNextPrevMenu(bool next);

    void setupEmptyMenu();
    void setupWindowMenu();
};

#endif /* MENUBARAPPLET_H */
