/*
 * Plasma applet to display application window menus
 *
 * Copyright 2010 Canonical Ltd.
 *
 * Authors:
 * - Aurélien Gâteau <aurelien.gateau@canonical.com>
 *
 * License: GPL v3
 */
// Self
#include "registrar.h"

// Qt
#include <QDBusMessage>
#include <QDBusObjectPath>
#include <QGraphicsView>
#include <QMenu>
#include <QTimer>

// KDE
#include <KDebug>
#include <KIcon>
#include <KWindowSystem>
#include <Plasma/Applet>

// dbusmenu-qt
#include <dbusmenuimporter.h>

class MyDBusMenuImporter : public DBusMenuImporter
{
public:
    MyDBusMenuImporter(const QString &service, const QString &path, QObject *parent)
    : DBusMenuImporter(service, path, parent)
    {}

protected:
    virtual QIcon iconForName(const QString &name)
    {
        return KIcon(name);
    }
};

Registrar::Registrar(Plasma::Applet *applet)
: QObject(applet)
, mActiveWinId(0)
, mApplet(applet)
{
    KWindowSystem* ws = KWindowSystem::self();
    connect(ws, SIGNAL(activeWindowChanged(WId)), SLOT(updateActiveWinId()));
    connect(ws, SIGNAL(windowRemoved(WId)), SLOT(slotWindowRemoved(WId)));

    updateActiveWinId();
}

void Registrar::RegisterWindow(int wid, const QDBusObjectPath& menuObjectPath)
{
    kDebug() << wid << message().service() << menuObjectPath.path();
    DBusMenuImporter* importer = new MyDBusMenuImporter(message().service(), menuObjectPath.path(), this);
    delete mImporters.take(wid);
    mImporters.insert(wid, importer);
    // This is slightly hackish: we simulate an aboutToShow event on the menu
    // so that we get the "menuReadyToBeShown" signal from DBusMenuImporter
    // when the menu has been filled.
    // The goal is to avoid showing an empty menu when the window has just been
    // registered: the menu would be empty because the importer hasn't had the
    // time to import it yet.
    connect(importer, SIGNAL(menuReadyToBeShown()), SLOT(slotMenuReadyToBeShown()));
    QMetaObject::invokeMethod(importer->menu(), "aboutToShow");
}

QMenu* Registrar::currentMenu() const
{
    if (!mActiveWinId) {
        return 0;
    }
    DBusMenuImporter* importer = mImporters.value(mActiveWinId, 0);
    return importer ? importer->menu() : 0;
}

void Registrar::slotWindowRemoved(WId id)
{
    delete mImporters.take(id);
}

void Registrar::updateActiveWinId()
{
    WId id = KWindowSystem::activeWindow();
    if (id == mActiveWinId) {
        return;
    }
    if (mApplet->view() && id == mApplet->view()->window()->winId()) {
        // Do not update id if the active window is the one hosting this applet
        return;
    }
    mActiveWinId = id;
    currentMenuChanged();
}

void Registrar::slotMenuReadyToBeShown()
{
    DBusMenuImporter* importer = static_cast<DBusMenuImporter*>(sender());
    disconnect(importer, SIGNAL(menuReadyToBeShown()), this, SLOT(slotMenuReadyToBeShown()));
    if (mActiveWinId && mImporters.value(mActiveWinId) == importer) {
        currentMenuChanged();
    }
}
