/*
 * Plasma applet to display application window menus
 *
 * Copyright 2010 Canonical Ltd.
 *
 * Authors:
 * - Aurélien Gâteau <aurelien.gateau@canonical.com>
 *
 * License: GPL v3
 */
#include <windowmenumanager.moc>

// Qt
#include <QX11Info>

// KDE
#include <KDebug>
#include <KIcon>
#include <KLocale>
#include <KWindowSystem>
#include <netwm.h>

WindowMenuManager::WindowMenuManager(QObject* parent)
: QObject(parent)
, mMenu(new QMenu)
, mWid(0)
{
    connect(mMenu, SIGNAL(destroyed()), SLOT(deleteLater()));
    connect(mMenu, SIGNAL(aboutToShow()), SLOT(updateActions()));

    mMinimizeAction = mMenu->addAction(QString(), this, SLOT(toggleMinimize()));
    mMaximizeAction = mMenu->addAction(QString(), this, SLOT(toggleMaximize()));
    mShadeAction = mMenu->addAction(QString(), this, SLOT(toggleShade()));
    mMenu->addSeparator();
    mCloseAction = mMenu->addAction(i18n("Close"), this, SLOT(closeWindow()));
    mCloseAction->setIcon(KIcon("window-close"));
}

WindowMenuManager::~WindowMenuManager()
{
    mMenu->deleteLater();
}

void WindowMenuManager::updateActions()
{
    if (!mWid) {
        kWarning() << "No winId!";
        return;
    }
    NETWinInfo ni(QX11Info::display(), mWid, QX11Info::appRootWindow(), NET::WMState);
    mMinimizeAction->setText(isMinimized() ? i18n("Restore") : i18n("Minimize"));
    mMaximizeAction->setText(ni.state() & NET::Max ? i18n("Unmaximize") : i18n("Maximize"));
    mShadeAction->setText(ni.state() & NET::Shaded ? i18n("Unshade") : i18n("Shade"));
}

void WindowMenuManager::setWinId(WId wid)
{
    mWid = wid;
}

void WindowMenuManager::toggleMaximize()
{
    if (isMinimized()) {
        KWindowSystem::unminimizeWindow(mWid);
    }
    toggleNETState(NET::Max);
}

void WindowMenuManager::toggleNETState(NET::State state)
{
    NETWinInfo ni(QX11Info::display(), mWid, QX11Info::appRootWindow(), NET::WMState);
    if (ni.state() & state) {
        ni.setState(0, state);
    } else {
        ni.setState(state, state);
    }
}

void WindowMenuManager::toggleMinimize()
{
    if (isMinimized()) {
        KWindowSystem::unminimizeWindow(mWid);
    } else {
        KWindowSystem::minimizeWindow(mWid);
    }
}

void WindowMenuManager::toggleShade()
{
    if (isMinimized()) {
        KWindowSystem::unminimizeWindow(mWid);
    }
    toggleNETState(NET::Shaded);
}

void WindowMenuManager::closeWindow()
{
    NETRootInfo ri( QX11Info::display(), NET::CloseWindow );
    ri.closeWindowRequest(mWid);
}

bool WindowMenuManager::isMinimized() const
{
    KWindowInfo info = KWindowSystem::windowInfo(mWid, NET::WMState | NET::XAWMState | NET::WMDesktop);
    return info.isMinimized();
}
